@@ -1,10 +1,10 @@
-use super::super::super::{DbXxx, DbXxxKeyType};
+use super::super::super::{DbMapKeyType, DbXxxBase, DbXxxObjectSafe};
 use super::super::{
     CheckFileDbMap, CountOfPerSize, FileDbParams, KeysCountStats, LengthStats, RecordSizeStats,
 };
 use super::semtype::*;
 use super::tr::{IdxNode, TreeNode};
-use super::{dat, idx};
+use super::{idx, key, val};
 use std::borrow::Borrow;
 use std::cell::RefCell;
 use std::cmp::Ordering;
@@ -13,40 +13,41 @@ use std::io::Result;
 use std::path::Path;
 use std::rc::Rc;
 
-#[cfg(feature = "key_cache")]
-use super::kc::KeyCacheTrait;
-
-#[cfg(feature = "key_cache")]
-use super::kc;
+#[cfg(feature = "htx")]
+use super::htx;
 
 #[derive(Debug)]
-pub struct FileDbXxxInner<KT: DbXxxKeyType> {
+pub struct FileDbXxxInner<KT: DbMapKeyType> {
     dirty: bool,
     //
-    dat_file: dat::DatFile<KT>,
+    key_file: key::KeyFile<KT>,
+    val_file: val::ValueFile<KT>,
     idx_file: idx::IdxFile,
-    //
-    #[cfg(feature = "key_cache")]
-    key_cache: kc::KeyCache<KT>,
+    #[cfg(feature = "htx")]
+    htx_file: htx::HtxFile,
     //
     _phantom: std::marker::PhantomData<KT>,
 }
 
-impl<KT: DbXxxKeyType> FileDbXxxInner<KT> {
+impl<KT: DbMapKeyType> FileDbXxxInner<KT> {
     pub(crate) fn open_with_params<P: AsRef<Path>>(
         path: P,
         ks_name: &str,
         params: FileDbParams,
     ) -> Result<FileDbXxxInner<KT>> {
-        let dat_file = dat::DatFile::open_with_params(&path, ks_name, KT::signature(), &params)?;
+        let key_file = key::KeyFile::open_with_params(&path, ks_name, KT::signature(), &params)?;
+        let val_file = val::ValueFile::open_with_params(&path, ks_name, KT::signature(), &params)?;
         let idx_file = idx::IdxFile::open_with_params(&path, ks_name, KT::signature(), &params)?;
+        #[cfg(feature = "htx")]
+        let htx_file = htx::HtxFile::open_with_params(&path, ks_name, KT::signature(), &params)?;
         //
         Ok(Self {
-            dat_file,
+            key_file,
+            val_file,
             idx_file,
+            #[cfg(feature = "htx")]
+            htx_file,
             dirty: false,
-            #[cfg(feature = "key_cache")]
-            key_cache: kc::KeyCache::new(),
             _phantom: std::marker::PhantomData,
         })
     }
@@ -57,91 +58,121 @@ impl<KT: DbXxxKeyType> FileDbXxxInner<KT> {
 }
 
 // for utils
-impl<KT: DbXxxKeyType> FileDbXxxInner<KT> {
-    #[cfg(feature = "key_cache")]
+impl<KT: DbMapKeyType> FileDbXxxInner<KT> {
     #[inline]
-    fn clear_key_cache(&mut self, record_offset: RecordOffset) {
-        self.key_cache.delete(&record_offset);
-    }
-    #[cfg(feature = "key_cache")]
-    #[inline]
-    fn _clear_key_cache_all(&mut self) {
-        self.key_cache.clear();
-    }
-    #[cfg(feature = "key_cache")]
-    #[inline]
-    pub fn load_key_string(&mut self, record_offset: RecordOffset) -> Result<Rc<KT>> {
-        debug_assert!(record_offset != RecordOffset::new(0));
-        let string = match self.key_cache.get(&record_offset) {
-            Some(s) => s,
-            None => {
-                let key = self.dat_file.read_record_only_key(record_offset)?;
-                self.key_cache.put(&record_offset, key)
-            }
-        };
-        Ok(string)
-    }
-    #[cfg(not(feature = "key_cache"))]
-    #[inline]
-    pub fn load_key_string(&mut self, record_offset: RecordOffset) -> Result<Rc<KT>> {
-        self.load_key_string_no_cache(record_offset)
-            .map(|a| Rc::new(a))
+    pub(crate) fn load_key_data(&self, record_offset: KeyRecordOffset) -> Result<KT> {
+        debug_assert!(!record_offset.is_zero());
+        self.key_file.read_record_only_key(record_offset)
     }
     #[inline]
-    pub fn load_key_string_no_cache(&self, record_offset: RecordOffset) -> Result<KT> {
-        debug_assert!(record_offset != RecordOffset::new(0));
-        self.dat_file.read_record_only_key(record_offset)
+    fn load_value(&self, record_offset: KeyRecordOffset) -> Result<Vec<u8>> {
+        debug_assert!(!record_offset.is_zero());
+        let value_offset = self.key_file.read_record_only_value_offset(record_offset)?;
+        self.val_file.read_record_only_value(value_offset)
     }
     #[inline]
-    fn load_value(&self, record_offset: RecordOffset) -> Result<Vec<u8>> {
-        debug_assert!(record_offset != RecordOffset::new(0));
-        self.dat_file.read_record_only_value(record_offset)
+    fn load_key_record_size(&self, record_offset: KeyRecordOffset) -> Result<KeyRecordSize> {
+        self.key_file.read_record_only_size(record_offset)
     }
     #[inline]
-    fn load_record_size(&self, record_offset: RecordOffset) -> Result<RecordSize> {
-        self.dat_file.read_record_only_size(record_offset)
+    fn load_value_record_size(&self, record_offset: KeyRecordOffset) -> Result<ValueRecordSize> {
+        let value_offset = self.key_file.read_record_only_value_offset(record_offset)?;
+        self.val_file.read_record_only_size(value_offset)
     }
     #[inline]
-    fn load_key_length(&self, record_offset: RecordOffset) -> Result<KeyLength> {
-        self.dat_file.read_record_only_key_length(record_offset)
+    fn load_key_length(&self, record_offset: KeyRecordOffset) -> Result<KeyLength> {
+        self.key_file.read_record_only_key_length(record_offset)
     }
     #[inline]
-    fn load_value_length(&self, record_offset: RecordOffset) -> Result<ValueLength> {
-        self.dat_file.read_record_only_value_length(record_offset)
+    fn load_value_length(&self, record_offset: KeyRecordOffset) -> Result<ValueLength> {
+        let value_offset = self.key_file.read_record_only_value_offset(record_offset)?;
+        self.val_file.read_record_only_value_length(value_offset)
     }
 
-    fn keys_binary_search<Q>(
+    #[cfg(any(feature = "vf_node_u32", feature = "vf_node_u64"))]
+    fn keys_binary_search_uu_kt(
         &mut self,
-        node: &TreeNode,
-        key: &Q,
-    ) -> Result<std::result::Result<usize, usize>>
-    where
-        KT: Borrow<Q> + Ord,
-        Q: Ord + ?Sized,
-    {
-        /*
-        match node.keys.binary_search_by(|&key_offset| {
-            debug_assert!(!key_offset.is_zero());
-            let key_string = self.load_key_string(key_offset).unwrap();
-            key_string.as_ref().borrow().cmp(key)
-        }) {
-            Ok(k) => Ok(Ok(k)),
-            Err(k) => Ok(Err(k)),
+        node_offset: NodeOffset,
+        key_kt: &KT,
+    ) -> Result<std::result::Result<KeyRecordOffset, NodeOffset>> {
+        #[cfg(feature = "vf_node_u32")]
+        const OFFSET_BYTE_SIZE: u32 = 4;
+        #[cfg(feature = "vf_node_u64")]
+        const OFFSET_BYTE_SIZE: u32 = 8;
+        //
+        let mut locked_key = self.key_file.0.borrow_mut();
+        let mut locked_idx = self.idx_file.0.borrow_mut();
+        //
+        let _ = locked_idx.0.seek_from_start(node_offset)?;
+        let _ = locked_idx.0.read_node_size()?;
+        let is_leaf = locked_idx.0.read_u8()?;
+        let keys_count = locked_idx.0.read_keys_count()?;
+        if keys_count.is_zero() {
+            return Ok(Err(NodeOffset::new(0)));
         }
-        */
+        let keys_count = keys_count.as_value() as u32;
+        let keys_start: NodeOffset = locked_idx.0.seek_position()?;
+        //
         let mut left = 0;
-        let mut right = node.keys_len();
+        let mut right = keys_count;
         while left < right {
             let mid = (left + right) / 2;
             //
             // SAFETY: `mid` is limited by `[left; right)` bound.
-            let key_offset = unsafe { node.keys_get_unchecked(mid) };
+            //let key_offset = node.keys[mid];
+            let _ = locked_idx
+                .0
+                .seek_from_start(keys_start + NodeSize::new(OFFSET_BYTE_SIZE * mid))?;
+            #[cfg(feature = "vf_node_u32")]
+            let key_offset: KeyRecordOffset = locked_idx.0.read_record_offset_u32()?;
+            #[cfg(feature = "vf_node_u64")]
+            let key_offset: KeyRecordOffset = locked_idx.0.read_record_offset_u64()?;
+            //
+            debug_assert!(!key_offset.is_zero());
+            let key_string = locked_key.read_record_only_key_maybeslice(key_offset)?;
+            match key_kt.cmp_u8(&key_string) {
+                Ordering::Greater => left = mid + 1,
+                Ordering::Equal => {
+                    return Ok(Ok(key_offset));
+                }
+                Ordering::Less => right = mid,
+            }
+        }
+        if is_leaf == 0 {
+            let _ = locked_idx.0.seek_from_start(
+                keys_start + NodeSize::new(OFFSET_BYTE_SIZE * (keys_count + left)),
+            )?;
+            #[cfg(feature = "vf_node_u32")]
+            let node_offset = locked_idx.0.read_node_offset_u32()?;
+            #[cfg(feature = "vf_node_u64")]
+            let node_offset = locked_idx.0.read_node_offset_u64()?;
+            Ok(Err(node_offset))
+        } else {
+            Ok(Err(NodeOffset::new(0)))
+        }
+    }
+    //
+    fn keys_binary_search_kt(
+        &mut self,
+        node: &TreeNode,
+        key_kt: &KT,
+    ) -> Result<std::result::Result<usize, usize>> {
+        let mut locked = self.key_file.0.borrow_mut();
+        //
+        let keys = node.keys();
+        let mut left = 0;
+        let mut right = keys.len();
+        while left < right {
+            let mid = (left + right) / 2;
+            //
+            // SAFETY: `mid` is limited by `[left; right)` bound.
+            //let key_offset = unsafe { node.keys_get_unchecked(mid) };
+            let key_offset = unsafe { *keys.get_unchecked(mid) };
             //let key_offset = node.keys[mid];
             //
-            debug_assert!(key_offset != RecordOffset::new(0));
-            let key_string = self.load_key_string(key_offset)?;
-            //
-            match key.borrow().cmp(key_string.as_ref().borrow()) {
+            debug_assert!(!key_offset.is_zero());
+            let key_string = locked.read_record_only_key_maybeslice(key_offset)?;
+            match key_kt.cmp_u8(&key_string) {
                 Ordering::Greater => left = mid + 1,
                 Ordering::Equal => {
                     return Ok(Ok(mid));
@@ -151,7 +182,7 @@ impl<KT: DbXxxKeyType> FileDbXxxInner<KT> {
         }
         Ok(Err(left))
     }
-
+    //
     #[inline]
     fn write_node(&mut self, node: IdxNode) -> Result<IdxNode> {
         self.dirty = true;
@@ -165,7 +196,11 @@ impl<KT: DbXxxKeyType> FileDbXxxInner<KT> {
 }
 
 // for debug
-impl<KT: DbXxxKeyType + std::fmt::Display> CheckFileDbMap for FileDbXxxInner<KT> {
+impl<KT: DbMapKeyType + std::fmt::Display> CheckFileDbMap for FileDbXxxInner<KT> {
+    #[cfg(feature = "htx")]
+    fn ht_size_and_count(&self) -> Result<(u64, u64)> {
+        self.htx_file.ht_size_and_count()
+    }
     /// convert the index node tree to graph string for debug.
     fn graph_string(&self) -> Result<String> {
         self.idx_file.graph_string()
@@ -198,14 +233,29 @@ impl<KT: DbXxxKeyType + std::fmt::Display> CheckFileDbMap for FileDbXxxInner<KT>
     fn count_of_free_node(&self) -> Result<CountOfPerSize> {
         self.idx_file.count_of_free_node()
     }
-    /// count of the free record
-    fn count_of_free_record(&self) -> Result<CountOfPerSize> {
-        self.dat_file.count_of_free_record()
+    /// count of the free key record
+    fn count_of_free_key_record(&self) -> Result<CountOfPerSize> {
+        self.key_file.count_of_free_key_record()
+    }
+    /// count of the free value record
+    fn count_of_free_value_record(&self) -> Result<CountOfPerSize> {
+        self.val_file.count_of_free_value_record()
     }
     /// count of the used record and the used node
-    fn count_of_used_node(&self) -> Result<(CountOfPerSize, CountOfPerSize)> {
-        self.idx_file
-            .count_of_used_node(|off| self.load_record_size(off))
+    fn count_of_used_node(&self) -> Result<(CountOfPerSize, CountOfPerSize, CountOfPerSize)> {
+        self.idx_file.count_of_used_node(|off| {
+            let ks = self.load_key_record_size(off);
+            if let Err(err) = ks {
+                Err(err)
+            } else {
+                let vs = self.load_value_record_size(off);
+                if let Err(err) = vs {
+                    Err(err)
+                } else {
+                    Ok((ks.unwrap(), vs.unwrap()))
+                }
+            }
+        })
     }
     /// buffer statistics
     #[cfg(feature = "buf_stats")]
@@ -216,9 +266,13 @@ impl<KT: DbXxxKeyType + std::fmt::Display> CheckFileDbMap for FileDbXxxInner<KT>
         vec
     }
     /// record size statistics
-    fn record_size_stats(&self) -> Result<RecordSizeStats> {
+    fn key_record_size_stats(&self) -> Result<RecordSizeStats<Key>> {
         self.idx_file
-            .record_size_stats(|off| self.load_record_size(off))
+            .record_size_stats(|off| self.load_key_record_size(off))
+    }
+    fn value_record_size_stats(&self) -> Result<RecordSizeStats<Value>> {
+        self.idx_file
+            .record_size_stats(|off| self.load_value_record_size(off))
     }
     /// keys count statistics
     fn keys_count_stats(&self) -> Result<KeysCountStats> {
@@ -237,24 +291,34 @@ impl<KT: DbXxxKeyType + std::fmt::Display> CheckFileDbMap for FileDbXxxInner<KT>
 }
 
 // insert: NEW
-impl<KT: DbXxxKeyType> FileDbXxxInner<KT> {
-    fn insert_into_node_tree(
+impl<KT: DbMapKeyType> FileDbXxxInner<KT> {
+    fn insert_into_node_tree_kt(
         &mut self,
         mut node_: IdxNode,
-        key: &KT,
+        key_kt: &KT,
         value: &[u8],
     ) -> Result<IdxNode> {
         let r = {
             let node = node_.get_ref();
             if node.keys_is_empty() {
-                let new_record = self.dat_file.add_record(key, value)?;
+                let new_val_record = self.val_file.add_value_record(value)?;
+                let new_key_record = self
+                    .key_file
+                    .add_key_record(key_kt, new_val_record.offset)?;
+                #[cfg(feature = "htx")]
+                {
+                    let off = new_key_record.offset;
+                    let hash = new_key_record.hash_value();
+                    self.htx_file.write_key_record_offset(hash, off)?;
+                }
+                //
                 return Ok(IdxNode::new_active(
-                    new_record.offset,
+                    new_key_record.offset,
                     NodeOffset::new(0),
                     NodeOffset::new(0),
                 ));
             }
-            self.keys_binary_search(&node, key)?
+            self.keys_binary_search_kt(&node, key_kt)?
         };
         match r {
             Ok(k) => {
@@ -263,6 +327,12 @@ impl<KT: DbXxxKeyType> FileDbXxxInner<KT> {
                 debug_assert!(record_offset != RecordOffset::new(0));
                 let new_record_offset = self.store_value_on_insert(record_offset, value)?;
                 if record_offset != new_record_offset {
+                    #[cfg(feature = "htx")]
+                    {
+                        let hash = key_kt.hash_value();
+                        self.htx_file
+                            .write_key_record_offset(hash, new_record_offset)?;
+                    }
                     node_.get_mut().keys_set(k, new_record_offset);
                     return self.write_node(node_);
                 }
@@ -273,10 +343,23 @@ impl<KT: DbXxxKeyType> FileDbXxxInner<KT> {
                 //let node_offset1 = node.downs[k];
                 let node2_ = if !node_offset1.is_zero() {
                     let node1_ = self.idx_file.read_node(node_offset1)?;
-                    self.insert_into_node_tree(node1_, key, value)?
+                    self.insert_into_node_tree_kt(node1_, key_kt, value)?
                 } else {
-                    let new_record = self.dat_file.add_record(key, value)?;
-                    IdxNode::new_active(new_record.offset, NodeOffset::new(0), NodeOffset::new(0))
+                    let new_val_record = self.val_file.add_value_record(value)?;
+                    let new_key_record = self
+                        .key_file
+                        .add_key_record(key_kt, new_val_record.offset)?;
+                    #[cfg(feature = "htx")]
+                    {
+                        let hash = key_kt.hash_value();
+                        self.htx_file
+                            .write_key_record_offset(hash, new_key_record.offset)?;
+                    }
+                    IdxNode::new_active(
+                        new_key_record.offset,
+                        NodeOffset::new(0),
+                        NodeOffset::new(0),
+                    )
                 };
                 if node2_.is_active_on_insert() {
                     self.balance_on_insert(node_, k, &node2_)
@@ -292,13 +375,16 @@ impl<KT: DbXxxKeyType> FileDbXxxInner<KT> {
     #[inline]
     fn store_value_on_insert(
         &mut self,
-        record_offset: RecordOffset,
+        record_offset: KeyRecordOffset,
         value: &[u8],
-    ) -> Result<RecordOffset> {
-        let mut record = self.dat_file.read_record(record_offset)?;
-        record.value = value.to_vec();
-        let new_record = self.dat_file.write_record(record)?;
-        Ok(new_record.offset)
+    ) -> Result<KeyRecordOffset> {
+        let mut key_record = self.key_file.read_record(record_offset)?;
+        let mut val_record = self.val_file.read_record(key_record.value_offset)?;
+        val_record.value = value.to_vec();
+        let new_value_record = self.val_file.write_record(val_record)?;
+        key_record.value_offset = new_value_record.offset;
+        let new_key_record = self.key_file.write_record(key_record)?;
+        Ok(new_key_record.offset)
     }
     #[inline]
     fn balance_on_insert(
@@ -357,71 +443,83 @@ impl<KT: DbXxxKeyType> FileDbXxxInner<KT> {
 }
 
 // delete: NEW
-impl<KT: DbXxxKeyType> FileDbXxxInner<KT> {
-    fn delete_from_node_tree<Q>(&mut self, mut node_: IdxNode, key: &Q) -> Result<IdxNode>
-    where
-        KT: Borrow<Q> + Ord,
-        Q: Ord + ?Sized,
-    {
-        if node_.get_ref().keys_is_empty() {
-            return Ok(node_);
-        }
-        let r = self.keys_binary_search(&node_.get_ref(), key)?;
+impl<KT: DbMapKeyType> FileDbXxxInner<KT> {
+    fn delete_from_node_tree_kt(
+        &mut self,
+        mut node_: IdxNode,
+        key_kt: &KT,
+    ) -> Result<(IdxNode, Option<Vec<u8>>)> {
+        let r = {
+            if node_.get_ref().keys_is_empty() {
+                return Ok((node_, None));
+            }
+            let node = node_.get_ref();
+            self.keys_binary_search_kt(&node, key_kt)?
+        };
         match r {
             Ok(k) => {
-                let node_ = self.delete_at(node_, k)?;
-                return Ok(node_);
+                let (node_, val) = self.delete_at(node_, k)?;
+                return Ok((node_, val));
             }
             Err(k) => {
                 let node_offset1 = unsafe { node_.get_mut().downs_get_unchecked(k) };
                 //let node_offset1 = node.downs[k];
                 if !node_offset1.is_zero() {
                     let node1_ = self.idx_file.read_node(node_offset1)?;
-                    let node1_ = self.delete_from_node_tree(node1_, key)?;
+                    let (node1_, val) = self.delete_from_node_tree_kt(node1_, key_kt)?;
                     node_.get_mut().downs_set(k, node1_.get_ref().offset());
                     let node_ = self.write_node(node_)?;
                     if k == node_.get_ref().downs_len() - 1 {
                         let node_ = self.balance_right(node_, k)?;
-                        return Ok(node_);
+                        return Ok((node_, val));
                     } else {
                         let node_ = self.balance_left(node_, k)?;
-                        return Ok(node_);
+                        return Ok((node_, val));
                     }
                 }
             }
         }
-        Ok(node_)
+        Ok((node_, None))
     }
     #[inline]
-    fn delete_at(&mut self, mut node_: IdxNode, i: usize) -> Result<IdxNode> {
+    fn delete_at(&mut self, mut node_: IdxNode, i: usize) -> Result<(IdxNode, Option<Vec<u8>>)> {
         let record_offset = node_.get_ref().keys_get(i);
         debug_assert!(
-            record_offset != RecordOffset::new(0),
+            !record_offset.is_zero(),
             "key_offset: {} != 0",
             record_offset
         );
-        {
-            #[cfg(feature = "key_cache")]
-            self.clear_key_cache(record_offset);
-            self.dat_file.delete_record(record_offset)?;
-        }
+        let opt_value = {
+            let record = self.key_file.read_record(record_offset)?;
+            #[cfg(feature = "htx")]
+            {
+                let hash = record.key.hash_value();
+                self.htx_file
+                    .write_key_record_offset(hash, KeyRecordOffset::new(0))?;
+            }
+            let value = self.val_file.read_record_only_value(record.value_offset)?;
+            self.val_file.delete_record(record.value_offset)?;
+            self.key_file.delete_record(record_offset)?;
+            Some(value)
+        };
         let node_offset1 = node_.get_ref().downs_get(i);
         if node_offset1.is_zero() {
             let _key_offset = node_.get_mut().keys_remove(i);
             let _node_offset = node_.get_mut().downs_remove(i);
             let new_node_ = self.write_node(node_)?;
-            Ok(new_node_)
+            Ok((new_node_, opt_value))
         } else {
             let node1_ = self.idx_file.read_node(node_offset1)?;
             let (record_offset, node1_) = self.delete_max(node1_)?;
             node_.get_mut().keys_set(i, record_offset);
             node_.get_mut().downs_set(i, node1_.get_ref().offset());
             let node_ = self.write_node(node_)?;
-            self.balance_left(node_, i)
+            let new_node_ = self.balance_left(node_, i)?;
+            Ok((new_node_, opt_value))
         }
     }
     #[inline]
-    fn delete_max(&mut self, mut node_: IdxNode) -> Result<(RecordOffset, IdxNode)> {
+    fn delete_max(&mut self, mut node_: IdxNode) -> Result<(KeyRecordOffset, IdxNode)> {
         let j = node_.get_ref().keys_len();
         let i = j - 1;
         let node_offset1 = node_.get_ref().downs_get(j);
@@ -531,10 +629,10 @@ impl<KT: DbXxxKeyType> FileDbXxxInner<KT> {
     #[inline]
     fn move_a_node_from_right_to_left(
         &mut self,
-        record_offset: RecordOffset,
+        record_offset: KeyRecordOffset,
         node_l: &mut IdxNode,
         node_r: &mut IdxNode,
-    ) -> RecordOffset {
+    ) -> KeyRecordOffset {
         node_l.get_mut().keys_push(record_offset);
         node_l
             .get_mut()
@@ -544,10 +642,10 @@ impl<KT: DbXxxKeyType> FileDbXxxInner<KT> {
     #[inline]
     fn move_left_right(
         &mut self,
-        record_offset: RecordOffset,
+        record_offset: KeyRecordOffset,
         node_l: &mut IdxNode,
         node_r: &mut IdxNode,
-    ) -> RecordOffset {
+    ) -> KeyRecordOffset {
         let j = node_l.get_ref().keys_len();
         let i = j - 1;
         node_r.get_mut().keys_insert(0, record_offset);
@@ -571,115 +669,49 @@ impl<KT: DbXxxKeyType> FileDbXxxInner<KT> {
 }
 
 // find: NEW
-impl<KT: DbXxxKeyType> FileDbXxxInner<KT> {
-    fn find_in_node_tree<Q>(&mut self, node_: &mut IdxNode, key: &Q) -> Result<Option<Vec<u8>>>
-    where
-        KT: Borrow<Q> + Ord,
-        Q: Ord + ?Sized,
-    {
-        let r = {
-            let node = node_.get_ref();
-            if node.keys_is_empty() {
-                return Ok(None);
-            }
-            self.keys_binary_search(&node, key)?
-        };
+impl<KT: DbMapKeyType> FileDbXxxInner<KT> {
+    fn find_in_node_tree_kt(
+        &mut self,
+        node_offset: NodeOffset,
+        key_kt: &KT,
+    ) -> Result<Option<Vec<u8>>> {
+        let r = self.keys_binary_search_uu_kt(node_offset, key_kt)?;
         match r {
-            Ok(k) => {
-                let key_offset = unsafe { node_.get_ref().keys_get_unchecked(k) };
-                //let key_offset = node_.get_ref().keys[k];
-                debug_assert!(key_offset != RecordOffset::new(0));
+            Ok(key_offset) => {
+                debug_assert!(!key_offset.is_zero());
                 self.load_value(key_offset).map(Some)
             }
-            Err(k) => {
-                let node_offset1 = unsafe { node_.get_ref().downs_get_unchecked(k) };
-                //let node_offset1 = node_.get_ref().downs[k];
-                if !node_offset1.is_zero() {
-                    let mut node1_ = self.idx_file.read_node(node_offset1)?;
-                    self.find_in_node_tree(&mut node1_, key)
+            Err(node_offset) => {
+                if !node_offset.is_zero() {
+                    self.find_in_node_tree_kt(node_offset, key_kt)
                 } else {
                     Ok(None)
                 }
             }
         }
     }
-    fn has_key_in_node_tree<Q>(&mut self, node_: &mut IdxNode, key: &Q) -> Result<bool>
-    where
-        KT: Borrow<Q> + Ord,
-        Q: Ord + ?Sized,
-    {
-        let r = {
-            let node = node_.get_ref();
-            if node.keys_is_empty() {
-                return Ok(false);
-            }
-            self.keys_binary_search(&node, key)?
-        };
-        match r {
-            Ok(_k) => Ok(true),
-            Err(k) => {
-                let node_offset1 = unsafe { node_.get_ref().downs_get_unchecked(k) };
-                //let node_offset1 = node_.get_ref().downs[k];
-                if !node_offset1.is_zero() {
-                    let mut node1_ = self.idx_file.read_node(node_offset1)?;
-                    self.has_key_in_node_tree(&mut node1_, key)
-                } else {
-                    Ok(false)
-                }
-            }
-        }
-    }
 }
 
-// impl trait: DbXxx<KT>
-impl<KT: DbXxxKeyType> DbXxx<KT> for FileDbXxxInner<KT> {
-    #[inline]
-    fn get<Q>(&mut self, key: &Q) -> Result<Option<Vec<u8>>>
-    where
-        KT: Borrow<Q> + Ord,
-        Q: Ord + ?Sized,
-    {
-        let mut top_node = self.idx_file.read_top_node()?;
-        self.find_in_node_tree(&mut top_node, key)
-    }
-    #[inline]
-    fn put(&mut self, key: KT, value: &[u8]) -> Result<()>
-    where
-        KT: Ord,
-    {
-        let top_node = self.idx_file.read_top_node()?;
-        let active_node = self.insert_into_node_tree(top_node, &key, value)?;
-        let new_top_node = active_node.deactivate();
-        self.idx_file.write_top_node(new_top_node)?;
-        Ok(())
-    }
-    #[inline]
-    fn delete<Q>(&mut self, key: &Q) -> Result<()>
-    where
-        KT: Borrow<Q> + Ord,
-        Q: Ord + ?Sized,
-    {
-        let top_node = self.idx_file.read_top_node()?;
-        let top_node_offset = top_node.get_ref().offset();
-        let top_node = self.delete_from_node_tree(top_node, key)?;
-        let new_top_node = self.trim(top_node)?;
-        if top_node_offset != new_top_node.get_ref().offset() {
-            self.idx_file.write_top_node(new_top_node)?;
-        }
-        Ok(())
-    }
+// impl trait: DbXxxBase
+impl<KT: DbMapKeyType> DbXxxBase for FileDbXxxInner<KT> {
     #[inline]
     fn read_fill_buffer(&mut self) -> Result<()> {
-        self.dat_file.read_fill_buffer()?;
+        self.val_file.read_fill_buffer()?;
+        self.key_file.read_fill_buffer()?;
         self.idx_file.read_fill_buffer()?;
+        #[cfg(feature = "htx")]
+        self.htx_file.read_fill_buffer()?;
         Ok(())
     }
     #[inline]
     fn flush(&mut self) -> Result<()> {
         if self.is_dirty() {
             // save all data
-            self.dat_file.flush()?;
+            self.val_file.flush()?;
+            self.key_file.flush()?;
             self.idx_file.flush()?;
+            #[cfg(feature = "htx")]
+            self.htx_file.flush()?;
             self.dirty = false;
         }
         Ok(())
@@ -688,8 +720,11 @@ impl<KT: DbXxxKeyType> DbXxx<KT> for FileDbXxxInner<KT> {
     fn sync_all(&mut self) -> Result<()> {
         if self.is_dirty() {
             // save all data and meta
-            self.dat_file.sync_all()?;
+            self.val_file.sync_all()?;
+            self.key_file.sync_all()?;
             self.idx_file.sync_all()?;
+            #[cfg(feature = "htx")]
+            self.htx_file.sync_all()?;
             self.dirty = false;
         }
         Ok(())
@@ -698,33 +733,114 @@ impl<KT: DbXxxKeyType> DbXxx<KT> for FileDbXxxInner<KT> {
     fn sync_data(&mut self) -> Result<()> {
         if self.is_dirty() {
             // save all data
-            self.dat_file.sync_data()?;
+            self.val_file.sync_data()?;
+            self.key_file.sync_data()?;
             self.idx_file.sync_data()?;
+            #[cfg(feature = "htx")]
+            self.htx_file.sync_data()?;
             self.dirty = false;
         }
         Ok(())
     }
+}
+
+// impl trait: DbXxxObjectSafe<KT>
+impl<KT: DbMapKeyType> DbXxxObjectSafe<KT> for FileDbXxxInner<KT> {
     #[inline]
-    fn has_key<Q>(&mut self, key: &Q) -> Result<bool>
-    where
-        KT: Borrow<Q> + Ord,
-        Q: Ord + ?Sized,
-    {
-        let mut top_node = self.idx_file.read_top_node()?;
-        self.has_key_in_node_tree(&mut top_node, &(*key.borrow()))
+    fn get_kt(&mut self, key_kt: &KT) -> Result<Option<Vec<u8>>> {
+        #[cfg(feature = "htx")]
+        {
+            let hash = key_kt.hash_value();
+            let key_offset = self.htx_file.read_key_record_offset(hash)?;
+            if !key_offset.is_zero() {
+                let flg = {
+                    let mut locked_key = self.key_file.0.borrow_mut();
+                    let key_string = locked_key.read_record_only_key_maybeslice(key_offset)?;
+                    match key_kt.cmp_u8(&key_string) {
+                        Ordering::Equal => true,
+                        Ordering::Greater => false,
+                        Ordering::Less => false,
+                    }
+                };
+                if flg {
+                    #[cfg(feature = "htx_print_hits")]
+                    self.htx_file.set_hits();
+                    return self.load_value(key_offset).map(Some);
+                } else {
+                    #[cfg(feature = "htx_print_hits")]
+                    self.htx_file.set_miss();
+                }
+            }
+        }
+        #[cfg(feature = "node_cache")]
+        {
+            let mut locked_idx = RefCell::borrow_mut(&self.idx_file.0);
+            locked_idx.flush_node_cache_clear()?
+        }
+        let node_offset = {
+            let mut locked_idx = self.idx_file.0.borrow_mut();
+            locked_idx.0.read_top_node_offset()?
+        };
+        self.find_in_node_tree_kt(node_offset, key_kt)
+    }
+    #[inline]
+    fn put_kt(&mut self, key_kt: &KT, value: &[u8]) -> Result<()> {
+        #[cfg(feature = "htx")]
+        {
+            let hash = key_kt.hash_value();
+            let key_offset = self.htx_file.read_key_record_offset(hash)?;
+            if !key_offset.is_zero() {
+                let flg = {
+                    let mut locked_key = self.key_file.0.borrow_mut();
+                    let key_string = locked_key.read_record_only_key_maybeslice(key_offset)?;
+                    match key_kt.cmp_u8(&key_string) {
+                        Ordering::Equal => true,
+                        Ordering::Greater => false,
+                        Ordering::Less => false,
+                    }
+                };
+                if flg {
+                    #[cfg(feature = "htx_print_hits")]
+                    self.htx_file.set_hits();
+                    //
+                    let new_record_offset = self.store_value_on_insert(key_offset, value)?;
+                    assert!(key_offset == new_record_offset);
+                    return Ok(());
+                } else {
+                    #[cfg(feature = "htx_print_hits")]
+                    self.htx_file.set_miss();
+                }
+            }
+        }
+        let top_node = self.idx_file.read_top_node()?;
+        let active_node = self.insert_into_node_tree_kt(top_node, key_kt, value)?;
+        let new_top_node = active_node.deactivate();
+        self.idx_file.write_top_node(new_top_node)?;
+        Ok(())
+    }
+    #[inline]
+    fn del_kt(&mut self, key_kt: &KT) -> Result<Option<Vec<u8>>> {
+        let top_node = self.idx_file.read_top_node()?;
+        let top_node_offset = top_node.get_ref().offset();
+        let (top_node, opt_val) = self.delete_from_node_tree_kt(top_node, key_kt)?;
+        let new_top_node = self.trim(top_node)?;
+        if top_node_offset != new_top_node.get_ref().offset() {
+            self.idx_file.write_top_node(new_top_node)?;
+        }
+        Ok(opt_val)
     }
 }
 
 // for Iterator
 //
 #[derive(Debug)]
-pub struct DbXxxIterMut<KT: DbXxxKeyType> {
+pub struct DbXxxIterMut<KT: DbMapKeyType> {
     db_map: Rc<RefCell<FileDbXxxInner<KT>>>,
     /// node depth of top node to leaf node.
     depth_nodes: Vec<(IdxNode, i32, i32)>,
 }
 
-impl<KT: DbXxxKeyType> DbXxxIterMut<KT> {
+impl<KT: DbMapKeyType> DbXxxIterMut<KT> {
     pub fn new(db_map: Rc<RefCell<FileDbXxxInner<KT>>>) -> Result<Self> {
         let depth_nodes = {
             let db_map_inner = RefCell::borrow(&db_map);
@@ -749,7 +865,7 @@ impl<KT: DbXxxKeyType> DbXxxIterMut<KT> {
             depth_nodes,
         })
     }
-    fn next_record_offset(&mut self) -> Option<RecordOffset> {
+    fn next_record_offset(&mut self) -> Option<KeyRecordOffset> {
         if self.depth_nodes.is_empty() {
             return None;
         }
@@ -834,14 +950,14 @@ impl<KT: DbXxxKeyType> DbXxxIterMut<KT> {
 }
 
 // impl trait: Iterator
-impl<KT: DbXxxKeyType> Iterator for DbXxxIterMut<KT> {
+impl<KT: DbMapKeyType> Iterator for DbXxxIterMut<KT> {
     type Item = (KT, Vec<u8>);
     fn next(&mut self) -> Option<(KT, Vec<u8>)> {
         if let Some(key_offset) = self.next_record_offset() {
-            let mut db_map_inner = RefCell::borrow_mut(&self.db_map);
-            let key_rc = db_map_inner.load_key_string(key_offset).unwrap();
+            let db_map_inner = RefCell::borrow_mut(&self.db_map);
+            let key = db_map_inner.load_key_data(key_offset).unwrap();
             let value_vec = db_map_inner.load_value(key_offset).unwrap();
-            Some((key_rc.as_ref().clone(), value_vec))
+            Some((key, value_vec))
         } else {
             None
         }
@@ -850,11 +966,11 @@ impl<KT: DbXxxKeyType> Iterator for DbXxxIterMut<KT> {
 
 //
 #[derive(Debug)]
-pub struct DbXxxIter<KT: DbXxxKeyType> {
+pub struct DbXxxIter<KT: DbMapKeyType> {
     iter: DbXxxIterMut<KT>,
 }
 
-impl<KT: DbXxxKeyType> DbXxxIter<KT> {
+impl<KT: DbMapKeyType> DbXxxIter<KT> {
     #[inline]
     pub fn new(db_map: Rc<RefCell<FileDbXxxInner<KT>>>) -> Result<Self> {
         Ok(Self {
@@ -864,7 +980,7 @@ impl<KT: DbXxxKeyType> DbXxxIter<KT> {
 }
 
 // impl trait: Iterator
-impl<KT: DbXxxKeyType> Iterator for DbXxxIter<KT> {
+impl<KT: DbMapKeyType> Iterator for DbXxxIter<KT> {
     type Item = (KT, Vec<u8>);
     #[inline]
     fn next(&mut self) -> Option<(KT, Vec<u8>)> {
@@ -874,11 +990,11 @@ impl<KT: DbXxxKeyType> Iterator for DbXxxIter<KT> {
 
 //
 #[derive(Debug)]
-pub struct DbXxxIntoIter<KT: DbXxxKeyType> {
+pub struct DbXxxIntoIter<KT: DbMapKeyType> {
     iter: DbXxxIterMut<KT>,
 }
 
-impl<KT: DbXxxKeyType> DbXxxIntoIter<KT> {
+impl<KT: DbMapKeyType> DbXxxIntoIter<KT> {
     #[inline]
     pub fn new(db_map: Rc<RefCell<FileDbXxxInner<KT>>>) -> Result<Self> {
         Ok(Self {
@@ -888,7 +1004,7 @@ impl<KT: DbXxxKeyType> DbXxxIntoIter<KT> {
 }
 
 // impl trait: Iterator
-impl<KT: DbXxxKeyType> Iterator for DbXxxIntoIter<KT> {
+impl<KT: DbMapKeyType> Iterator for DbXxxIntoIter<KT> {
     type Item = (KT, Vec<u8>);
     #[inline]
     fn next(&mut self) -> Option<(KT, Vec<u8>)> {
