@@ -1,4 +1,4 @@
-use super::super::super::DbXxxKeyType;
+use super::super::super::DbMapKeyType;
 use super::super::{
     CountOfPerSize, FileBufSizeParam, FileDbParams, KeysCountStats, LengthStats, RecordSizeStats,
 };
@@ -15,12 +15,10 @@ use std::rc::Rc;
 
 type HeaderSignature = [u8; 8];
 
-const CHUNK_SIZE: u32 = 4 * 1024;
-//const CHUNK_SIZE: u32 = 2 * 4 * 1024;
-//const CHUNK_SIZE: u32 = 16 * 4 * 1024;
-//const CHUNK_SIZE: u32 = 1024 * 1024;
+//const CHUNK_SIZE: u32 = 4 * 1024;
+const CHUNK_SIZE: u32 = 4 * 4 * 1024;
 const IDX_HEADER_SZ: u64 = 128;
-const IDX_HEADER_SIGNATURE: HeaderSignature = [b's', b'i', b'a', b'm', b'd', b'b', b'1', 0u8];
+const IDX_HEADER_SIGNATURE: HeaderSignature = [b's', b'i', b'a', b'm', b'd', b'b', b'T', 0u8];
 const IDX_HEADER_TOP_NODE_OFFSET: u64 = 16;
 
 #[cfg(not(feature = "node_cache"))]
@@ -31,14 +29,14 @@ use super::nc::NodeCache;
 
 #[cfg(not(feature = "node_cache"))]
 #[derive(Debug)]
-struct VarFileNodeCache(VarFile, PhantomData<i32>);
+pub struct VarFileNodeCache(pub VarFile, PhantomData<i32>);
 
 #[cfg(feature = "node_cache")]
 #[derive(Debug)]
-struct VarFileNodeCache(VarFile, NodeCache);
+pub struct VarFileNodeCache(pub VarFile, NodeCache);
 
 #[derive(Debug, Clone)]
-pub struct IdxFile(Rc<RefCell<VarFileNodeCache>>);
+pub struct IdxFile(pub Rc<RefCell<VarFileNodeCache>>);
 
 impl IdxFile {
     pub fn open_with_params<P: AsRef<Path>>(
@@ -208,7 +206,7 @@ impl IdxFile {
     }
     pub fn graph_string_with_key_string<KT>(&self, dbxxx: &FileDbXxxInner<KT>) -> Result<String>
     where
-        KT: DbXxxKeyType + std::fmt::Display,
+        KT: DbMapKeyType + std::fmt::Display,
     {
         let top_node = self.read_top_node()?;
         let mut locked = RefCell::borrow_mut(&self.0);
@@ -269,14 +267,14 @@ impl IdxFile {
     //
     pub fn is_mst_valid<KT>(&self, node: &IdxNode, dbxxx: &FileDbXxxInner<KT>) -> Result<bool>
     where
-        KT: DbXxxKeyType + std::fmt::Display + std::default::Default + std::cmp::PartialOrd,
+        KT: DbMapKeyType + std::fmt::Display + std::default::Default + std::cmp::PartialOrd,
     {
         if node.get_ref().keys_is_empty() {
             return Ok(true);
         }
         let record_offset = node.get_ref().keys_get(0);
         let key_string = if !record_offset.is_zero() {
-            dbxxx.load_key_string_no_cache(record_offset)?
+            dbxxx.load_key_data(record_offset)?
         } else {
             Default::default()
         };
@@ -296,12 +294,12 @@ impl IdxFile {
             let key_offset2 = node.get_ref().keys_get(i);
             let node_offset = node.get_ref().downs_get(i);
             let key_string1 = if !key_offset1.is_zero() {
-                dbxxx.load_key_string_no_cache(key_offset1)?
+                dbxxx.load_key_data(key_offset1)?
             } else {
                 Default::default()
             };
             let key_string2 = if !key_offset2.is_zero() {
-                dbxxx.load_key_string_no_cache(key_offset2)?
+                dbxxx.load_key_data(key_offset2)?
             } else {
                 Default::default()
             };
@@ -324,7 +322,7 @@ impl IdxFile {
         if !node_offset.is_zero() {
             let node1 = self.read_node(node_offset)?;
             if !record_offset.is_zero() {
-                let key_string = dbxxx.load_key_string_no_cache(record_offset)?;
+                let key_string = dbxxx.load_key_data(record_offset)?;
                 if !self.is_large(&key_string, &node1, dbxxx)? {
                     return Ok(false);
                 }
@@ -339,7 +337,7 @@ impl IdxFile {
     //
     fn is_small<KT>(&self, key: &KT, node: &IdxNode, dbxxx: &FileDbXxxInner<KT>) -> Result<bool>
     where
-        KT: DbXxxKeyType + std::fmt::Display + std::default::Default + std::cmp::PartialOrd,
+        KT: DbMapKeyType + std::fmt::Display + std::default::Default + std::cmp::PartialOrd,
     {
         for i in 0..node.get_ref().keys_len() {
             let node_offset = node.get_ref().downs_get(i);
@@ -351,7 +349,7 @@ impl IdxFile {
             }
             let record_offset = node.get_ref().keys_get(i);
             if !record_offset.is_zero() {
-                let key_string1 = dbxxx.load_key_string_no_cache(record_offset)?;
+                let key_string1 = dbxxx.load_key_data(record_offset)?;
                 if key <= &key_string1 {
                     return Ok(false);
                 }
@@ -376,7 +374,7 @@ impl IdxFile {
         dbxxx: &FileDbXxxInner<KT>,
     ) -> Result<bool>
     where
-        KT: DbXxxKeyType + std::fmt::Display + std::default::Default + std::cmp::PartialOrd,
+        KT: DbMapKeyType + std::fmt::Display + std::default::Default + std::cmp::PartialOrd,
     {
         for i in 0..node.get_ref().keys_len() {
             let node_offset = node.get_ref().downs_get(i);
@@ -388,7 +386,7 @@ impl IdxFile {
             }
             let record_offset11 = node.get_ref().keys_get(i);
             if !record_offset11.is_zero() {
-                let ket_string11 = dbxxx.load_key_string_no_cache(record_offset11)?;
+                let ket_string11 = dbxxx.load_key_data(record_offset11)?;
                 if key1 >= &ket_string11 {
                     return Ok(false);
                 }
@@ -410,7 +408,7 @@ impl IdxFile {
     }
     fn is_large<KT>(&self, key: &KT, node: &IdxNode, dbxxx: &FileDbXxxInner<KT>) -> Result<bool>
     where
-        KT: DbXxxKeyType + std::fmt::Display + std::default::Default + std::cmp::PartialOrd,
+        KT: DbMapKeyType + std::fmt::Display + std::default::Default + std::cmp::PartialOrd,
     {
         for i in 0..node.get_ref().keys_len() {
             let node_offset = node.get_ref().downs_get(i);
@@ -422,7 +420,7 @@ impl IdxFile {
             }
             let record_offset = node.get_ref().keys_get(i);
             if !record_offset.is_zero() {
-                let ket_string1 = dbxxx.load_key_string_no_cache(record_offset)?;
+                let ket_string1 = dbxxx.load_key_data(record_offset)?;
                 if key >= &ket_string1 {
                     return Ok(false);
                 }
@@ -507,10 +505,10 @@ impl IdxFile {
     }
     pub fn count_of_used_node<F>(
         &self,
-        read_record_size_func: F,
-    ) -> Result<(CountOfPerSize, CountOfPerSize)>
+        read_key_value_record_size_func: F,
+    ) -> Result<(CountOfPerSize, CountOfPerSize, CountOfPerSize)>
     where
-        F: Fn(RecordOffset) -> Result<RecordSize> + std::marker::Copy,
+        F: Fn(KeyRecordOffset) -> Result<(KeyRecordSize, ValueRecordSize)> + std::marker::Copy,
     {
         let mut node_vec = Vec::new();
         for node_size in NODE_SIZE_ARY {
@@ -518,10 +516,15 @@ impl IdxFile {
             node_vec.push((node_size, cnt));
         }
         //
-        let mut record_vec = Vec::new();
-        for record_size in super::dat::REC_SIZE_ARY {
+        let mut key_record_vec = Vec::new();
+        for record_size in super::key::REC_SIZE_ARY {
             let cnt = 0;
-            record_vec.push((record_size, cnt));
+            key_record_vec.push((record_size, cnt));
+        }
+        let mut value_record_vec = Vec::new();
+        for record_size in super::val::REC_SIZE_ARY {
+            let cnt = 0;
+            value_record_vec.push((record_size, cnt));
         }
         //
         let top_node = self.read_top_node()?;
@@ -529,15 +532,17 @@ impl IdxFile {
         locked.count_of_used_node(
             &top_node,
             &mut node_vec,
-            &mut record_vec,
-            read_record_size_func,
+            &mut key_record_vec,
+            &mut value_record_vec,
+            read_key_value_record_size_func,
         )?;
         //
-        Ok((record_vec, node_vec))
+        Ok((key_record_vec, value_record_vec, node_vec))
     }
-    pub fn record_size_stats<F>(&self, read_record_size_func: F) -> Result<RecordSizeStats>
+    pub fn record_size_stats<KV, F>(&self, read_record_size_func: F) -> Result<RecordSizeStats<KV>>
     where
-        F: Fn(RecordOffset) -> Result<RecordSize> + std::marker::Copy,
+        F: Fn(KeyRecordOffset) -> Result<RecordSize<KV>> + std::marker::Copy,
+        KV: Default + Copy + Ord,
     {
         let mut record_size_stats = RecordSizeStats::default();
         //
@@ -561,7 +566,7 @@ impl IdxFile {
         read_key_length_func: F,
     ) -> Result<LengthStats<KV>>
     where
-        F: Fn(RecordOffset) -> Result<Length<KV>> + std::marker::Copy,
+        F: Fn(KeyRecordOffset) -> Result<Length<KV>> + std::marker::Copy,
     {
         let mut kv_length_stats = LengthStats::<KV>::default();
         //
@@ -584,8 +589,7 @@ The db index header size is 128 bytes.
 +--------+-------+-------------+---------------------------+
 | offset | bytes | name        | comment                   |
 +--------+-------+-------------+---------------------------+
-| 0      | 4     | signature1  | [b's', b'h', b'a', b'm']  |
-| 4      | 4     | signature1  | [b'd', b'b', b'1', 0u8]   |
+| 0      | 8     | signature1  | b"siamdbT\0"              |
 | 8      | 8     | signature2  | 8 bytes type signature    |
 | 16     | 8     | top node    | offset of top node        |
 | 24     | 8     | free1 off   | offset of free 1st list   |
@@ -639,7 +643,7 @@ impl VarFile {
         //
         Ok(())
     }
-    fn read_top_node_offset(&mut self) -> Result<NodeOffset> {
+    pub fn read_top_node_offset(&mut self) -> Result<NodeOffset> {
         self.seek_from_start(NodeOffset::new(IDX_HEADER_TOP_NODE_OFFSET))?;
         self.read_u64_le().map(NodeOffset::new)
     }
@@ -680,11 +684,44 @@ const NODE_SIZE_ARY: [u32; 8] = [
 
 #[cfg(not(feature = "small_node_slots"))]
 #[cfg(feature = "vf_u32u32")]
+//pub const NODE_SLOTS_MAX: u16 = 1023;
+//pub const NODE_SLOTS_MAX: u16 = 511;
+//pub const NODE_SLOTS_MAX: u16 = 255;
+//pub const NODE_SLOTS_MAX: u16 = 127;
+//pub const NODE_SLOTS_MAX: u16 = 63;
+//pub const NODE_SLOTS_MAX: u16 = 31;
+//pub const NODE_SLOTS_MAX: u16 = 24;
+pub const NODE_SLOTS_MAX: u16 = 12;
+
+#[cfg(not(feature = "small_node_slots"))]
+#[cfg(feature = "vf_u32u32")]
+const NODE_SIZE_ARY: [u32; 8] = [
+    16 * 2 * 2,
+    16 * 2 * 3,
+    16 * 2 * 4,
+    16 * 2 * 5,
+    16 * 2 * 6,
+    16 * 2 * 7,
+    16 * 2 * 8,
+    16 * 2 * 9,
+];
+
+#[cfg(not(feature = "small_node_slots"))]
+#[cfg(feature = "vf_u64u64")]
 pub const NODE_SLOTS_MAX: u16 = 12;
 
 #[cfg(not(feature = "small_node_slots"))]
 #[cfg(feature = "vf_u64u64")]
-pub const NODE_SLOTS_MAX: u16 = 6;
+const NODE_SIZE_ARY: [u32; 8] = [
+    16 * 3,
+    16 * 4,
+    16 * 6,
+    16 * 8,
+    16 * 10,
+    16 * 14,
+    16 * 16,
+    16 * 18,
+];
 
 #[cfg(not(feature = "small_node_slots"))]
 #[cfg(feature = "vf_vu64")]
@@ -695,6 +732,7 @@ pub const NODE_SLOTS_MAX: u16 = 6;
 db_map.depth_of_node_tree(): 4
 
 pub const NODE_SLOTS_MAX: u16 = 256;
+
 #[cfg(not(feature = "small_node_slots"))]
 const NODE_SIZE_ARY: [u32; 8] = [
     16 * 40,
@@ -732,7 +770,8 @@ const NODE_SIZE_ARY: [u32; 8] = [
 3824inputs+2171664outputs (4major+41444minor)pagefaults 0swaps
 383M	./cmp_siamesedb/target/bench-db.siamesedb
 db_map.depth_of_node_tree(): 5
-
+*/
+/*
 pub const NODE_SLOTS_MAX: u16 = 64;
 
 #[cfg(not(feature = "small_node_slots"))]
@@ -805,18 +844,20 @@ const NODE_SIZE_ARY: [u32; 8] = [
 381M	./cmp_siamesedb/target/bench-db.siamesedb
 db_map.depth_of_node_tree(): 8
 */
+/* ** */
 pub const NODE_SLOTS_MAX: u16 = 12;
 
 #[cfg(not(feature = "small_node_slots"))]
+#[cfg(feature = "vf_vu64")]
 const NODE_SIZE_ARY: [u32; 8] = [
-    16 * 2,
     16 * 3,
     16 * 4,
-    16 * 5,
     16 * 6,
-    16 * 7,
     16 * 8,
-    16 * 9,
+    16 * 10,
+    16 * 14,
+    16 * 16,
+    16 * 18,
 ];
 
 /*
@@ -894,7 +935,7 @@ impl VarFileNodeCache {
 
     #[cfg(feature = "node_cache")]
     #[inline]
-    fn flush_node_cache_clear(&mut self) -> Result<()> {
+    pub fn flush_node_cache_clear(&mut self) -> Result<()> {
         self.1.clear(&mut self.0)?;
         Ok(())
     }
@@ -1027,6 +1068,7 @@ impl VarFileNodeCache {
             "!node_size.is_zero(), offset: {}",
             offset
         );
+        let is_leaf = self.0.read_u8()?;
         let keys_count = self.0.read_keys_count()?;
         debug_assert!(
             keys_count.as_value() < NODE_SLOTS_MAX,
@@ -1039,25 +1081,53 @@ impl VarFileNodeCache {
         {
             let mut node = node_.get_mut();
             for _i in 0..keys_count {
+                #[cfg(not(any(feature = "vf_node_u32", feature = "vf_node_u64")))]
                 let record_offset = self
                     .0
                     .read_record_offset()
                     .unwrap_or_else(|_| panic!("offset:{}, i:{}", offset, _i));
+                #[cfg(feature = "vf_node_u32")]
+                let record_offset = self
+                    .0
+                    .read_record_offset_u32()
+                    .unwrap_or_else(|_| panic!("offset:{}, i:{}", offset, _i));
+                #[cfg(feature = "vf_node_u64")]
+                let record_offset = self
+                    .0
+                    .read_record_offset_u64()
+                    .unwrap_or_else(|_| panic!("offset:{}, i:{}", offset, _i));
                 debug_assert!(!record_offset.is_zero());
                 node.keys_push(record_offset);
             }
-            for _i in 0..(keys_count + 1) {
-                let node_offset = self
-                    .0
-                    .read_node_offset()
-                    .unwrap_or_else(|_| panic!("offset:{}, i:{}", offset, _i));
-                debug_assert!(
-                    (node_offset.as_value() & 0x0F) == 0,
-                    "(node_offset.as_value(): {} & 0x0F) == 0, offset: {}",
-                    node_offset,
-                    offset.as_value()
-                );
-                node.downs_push(node_offset);
+            if is_leaf == 0 {
+                for _i in 0..(keys_count + 1) {
+                    #[cfg(not(any(feature = "vf_node_u32", feature = "vf_node_u64")))]
+                    let node_offset = self
+                        .0
+                        .read_node_offset()
+                        .unwrap_or_else(|_| panic!("offset:{}, i:{}", offset, _i));
+                    #[cfg(feature = "vf_node_u32")]
+                    let node_offset = self
+                        .0
+                        .read_node_offset_u32()
+                        .unwrap_or_else(|_| panic!("offset:{}, i:{}", offset, _i));
+                    #[cfg(feature = "vf_node_u64")]
+                    let node_offset = self
+                        .0
+                        .read_node_offset_u64()
+                        .unwrap_or_else(|_| panic!("offset:{}, i:{}", offset, _i));
+                    debug_assert!(
+                        (node_offset.as_value() & 0x0F) == 0,
+                        "(node_offset.as_value(): {} & 0x0F) == 0, offset: {}",
+                        node_offset,
+                        offset.as_value()
+                    );
+                    node.downs_push(node_offset);
+                }
+            } else {
+                for _i in 0..(keys_count + 1) {
+                    node.downs_push(NodeOffset::new(0));
+                }
             }
         }
         debug_assert!(_start_pos + node_size >= self.0.seek_position()?);
@@ -1081,6 +1151,7 @@ impl VarFileNodeCache {
             "!node_size.is_zero(), offset: {}",
             offset
         );
+        let is_leaf = self.0.read_u8()?;
         let keys_count = self.0.read_keys_count()?;
         debug_assert!(
             keys_count.as_value() < NODE_SLOTS_MAX,
@@ -1093,20 +1164,48 @@ impl VarFileNodeCache {
         {
             let mut node = node_.get_mut();
             for _i in 0..keys_count {
+                #[cfg(not(any(feature = "vf_node_u32", feature = "vf_node_u64")))]
                 let record_offset = self
                     .0
                     .read_record_offset()
                     .unwrap_or_else(|_| panic!("offset:{}, i:{}", offset, _i));
+                #[cfg(feature = "vf_node_u32")]
+                let record_offset = self
+                    .0
+                    .read_record_offset_u32()
+                    .unwrap_or_else(|_| panic!("offset:{}, i:{}", offset, _i));
+                #[cfg(feature = "vf_node_u64")]
+                let record_offset = self
+                    .0
+                    .read_record_offset_u64()
+                    .unwrap_or_else(|_| panic!("offset:{}, i:{}", offset, _i));
                 debug_assert!(!record_offset.is_zero());
                 node.keys_push(record_offset);
             }
-            for _i in 0..(keys_count + 1) {
-                let node_offset = self
-                    .0
-                    .read_node_offset()
-                    .unwrap_or_else(|_| panic!("offset:{}, i:{}", offset, _i));
-                debug_assert!((node_offset.as_value() & 0x0F) == 0);
-                node.downs_push(node_offset);
+            if is_leaf == 0 {
+                for _i in 0..(keys_count + 1) {
+                    #[cfg(not(any(feature = "vf_node_u32", feature = "vf_node_u64")))]
+                    let node_offset = self
+                        .0
+                        .read_node_offset()
+                        .unwrap_or_else(|_| panic!("offset:{}, i:{}", offset, _i));
+                    #[cfg(feature = "vf_node_u32")]
+                    let node_offset = self
+                        .0
+                        .read_node_offset_u32()
+                        .unwrap_or_else(|_| panic!("offset:{}, i:{}", offset, _i));
+                    #[cfg(feature = "vf_node_u64")]
+                    let node_offset = self
+                        .0
+                        .read_node_offset_u64()
+                        .unwrap_or_else(|_| panic!("offset:{}, i:{}", offset, _i));
+                    debug_assert!((node_offset.as_value() & 0x0F) == 0);
+                    node.downs_push(node_offset);
+                }
+            } else {
+                for _i in 0..(keys_count + 1) {
+                    node.downs_push(NodeOffset::new(0));
+                }
             }
         }
         //
@@ -1167,7 +1266,7 @@ impl VarFileNodeCache {
         dbxxx: &FileDbXxxInner<KT>,
     ) -> Result<String>
     where
-        KT: DbXxxKeyType + std::fmt::Display,
+        KT: DbMapKeyType + std::fmt::Display,
     {
         let node = node_.get_ref();
         let mut gs = format!(
@@ -1190,7 +1289,7 @@ impl VarFileNodeCache {
             i -= 1;
             let record_offset = node.keys_get(i);
             if !record_offset.is_zero() {
-                let key_string = dbxxx.load_key_string_no_cache(record_offset)?;
+                let key_string = dbxxx.load_key_data(record_offset)?;
                 gs += &format!(
                     "{}{:04x}:'{}'\n",
                     head,
@@ -1217,11 +1316,12 @@ impl VarFileNodeCache {
         &mut self,
         node_: &IdxNode,
         node_vec: &mut Vec<(u32, u64)>,
-        record_vec: &mut Vec<(u32, u64)>,
-        read_record_size_func: F,
+        key_record_vec: &mut Vec<(u32, u64)>,
+        value_record_vec: &mut Vec<(u32, u64)>,
+        read_key_value_record_size_func: F,
     ) -> Result<()>
     where
-        F: Fn(RecordOffset) -> Result<RecordSize> + Copy,
+        F: Fn(KeyRecordOffset) -> Result<(KeyRecordSize, ValueRecordSize)> + std::marker::Copy,
     {
         let node = node_.get_ref();
         match node_vec.iter().position(|v| v.0 == node.size().as_value()) {
@@ -1240,24 +1340,43 @@ impl VarFileNodeCache {
             let node = self
                 .read_node(node_offset)
                 .unwrap_or_else(|_| panic!("offset: {:04x}", node_offset.as_value()));
-            self.count_of_used_node(&node, node_vec, record_vec, read_record_size_func)?;
+            self.count_of_used_node(
+                &node,
+                node_vec,
+                key_record_vec,
+                value_record_vec,
+                read_key_value_record_size_func,
+            )?;
         }
         while i > 0 {
             i -= 1;
             //
             let record_offset = node.keys_get(i);
             if !record_offset.is_zero() {
-                let record_size = read_record_size_func(record_offset)?;
-                match record_vec
+                let (key_record_size, value_record_size) =
+                    read_key_value_record_size_func(record_offset)?;
+                match key_record_vec
                     .iter()
-                    .position(|v| v.0 == record_size.as_value())
+                    .position(|v| v.0 == key_record_size.as_value())
                 {
                     Some(sz_idx) => {
-                        record_vec[sz_idx].1 += 1;
+                        key_record_vec[sz_idx].1 += 1;
                     }
                     None => {
-                        let last = record_vec.len() - 1;
-                        record_vec[last].1 += 1;
+                        let last = key_record_vec.len() - 1;
+                        key_record_vec[last].1 += 1;
+                    }
+                }
+                match value_record_vec
+                    .iter()
+                    .position(|v| v.0 == value_record_size.as_value())
+                {
+                    Some(sz_idx) => {
+                        value_record_vec[sz_idx].1 += 1;
+                    }
+                    None => {
+                        let last = value_record_vec.len() - 1;
+                        value_record_vec[last].1 += 1;
                     }
                 }
             }
@@ -1267,21 +1386,28 @@ impl VarFileNodeCache {
                 let node = self
                     .read_node(node_offset)
                     .unwrap_or_else(|_| panic!("offset: {:04x}", node_offset.as_value()));
-                self.count_of_used_node(&node, node_vec, record_vec, read_record_size_func)?;
+                self.count_of_used_node(
+                    &node,
+                    node_vec,
+                    key_record_vec,
+                    value_record_vec,
+                    read_key_value_record_size_func,
+                )?;
             }
         }
         //
         Ok(())
     }
 
-    fn idx_record_size_stats<F>(
+    fn idx_record_size_stats<KV, F>(
         &mut self,
         node_: &IdxNode,
-        record_vec: &mut RecordSizeStats,
+        record_vec: &mut RecordSizeStats<KV>,
         read_record_size_func: F,
     ) -> Result<()>
     where
-        F: Fn(RecordOffset) -> Result<RecordSize> + Copy,
+        F: Fn(KeyRecordOffset) -> Result<RecordSize<KV>> + Copy,
+        KV: Copy + Ord,
     {
         let node = node_.get_ref();
         let mut i = node.downs_len() - 1;
@@ -1351,7 +1477,7 @@ impl VarFileNodeCache {
         read_kv_length_func: F,
     ) -> Result<()>
     where
-        F: Fn(RecordOffset) -> Result<Length<KV>> + Copy,
+        F: Fn(KeyRecordOffset) -> Result<Length<KV>> + Copy,
     {
         let node = node_.get_ref();
         let mut i = node.downs_len() - 1;
@@ -1390,7 +1516,7 @@ impl VarFileNodeCache {
 
 /*
 ```text
-used node:
+used node piece:
 +--------+-------+-------------+-----------------------------------+
 | offset | bytes | name        | comment                           |
 +--------+-------+-------------+-----------------------------------+
@@ -1407,7 +1533,7 @@ used node:
 */
 /*
 ```text
-free node:
+free piece:
 +--------+-------+-------------+-----------------------------------+
 | offset | bytes | name        | comment                           |
 +--------+-------+-------------+-----------------------------------+
