@@ -5,25 +5,39 @@ The simple local key-value store.
 
 - key-value store.
 - in-memory and file store.
-- `DbMapString` has keys as utf-8 string.
-- `DbMapU64` has keys as u64.
+- `DbMapDbString` has keys as utf-8 string.
+- `DbMapDbInt` has keys as u64.
+- `DbMapDbBytes` has keys as Vec<u8>.
 - The value is any bytes included utf-8 string.
-- The file store is implemented the basic B-Tree. (no hash and no leaf)
+- The file store is implemented the basic B-Tree. (no leaf)
+- The file store is included the htx file that is hash cache table for performance.
 - Small db file size.
-- Separated files. (data record file and index file)
-- One database has some db-map-string and some db-map-u64.
+- Separated files. (key file, value file, index file and htx file)
+- One database has some db-map-string and some db-map-int and some db-map-bytes.
+- Swiss army knife with easy-to-use and good performance
 - minimum support rustc rustc 1.53.0 (53cb7b09b 2021-06-17)
 
 # Compatibility
 
 - Nothing?
 
+# Todo
+
+- [ ] more performance
+- [ ] DB lock as support for multi-process-safe
+
+# Low priority todo
+
+- [ ] transaction support that handles multiple key-space at a time.
+- [ ] thread-safe support
+- [ ] non db lock multi-process-safe support
+
 # Examples
 
-## Example DbMapString:
+## Example DbMapDbString:
 
 ```
-use siamesedb::{DbMapString, DbXxx};
+use siamesedb::{DbMapDbString, DbXxx, DbXxxBase, DbXxxObjectSafe};
 
 fn main() -> std::io::Result<()> {
     let db_name = "target/tmp/doc-test1.siamesedb";
@@ -36,18 +50,18 @@ fn main() -> std::io::Result<()> {
     //
     let r = db_map.get_string("key1")?;
     assert_eq!(r, None);
-    db_map.put_string("key1".to_string(), "value1")?;
+    db_map.put_string("key1".into(), "value1")?;
     let r = db_map.get_string("key1")?;
-    assert_eq!(r, Some("value1".to_string()));
+    assert_eq!(r, Some("value1".into()));
     db_map.sync_data()?;
     Ok(())
 }
 ```
 
-## Example DbMapU64:
+## Example DbMapDbInt:
 
 ```
-use siamesedb::{DbMapU64, DbXxx};
+use siamesedb::{DbMapDbInt, DbXxx, DbXxxBase, DbXxxObjectSafe};
 
 fn main() -> std::io::Result<()> {
     let db_name = "target/tmp/doc-test2.siamesedb";
@@ -55,10 +69,10 @@ fn main() -> std::io::Result<()> {
     let _ = std::fs::remove_dir_all(db_name);
     // create or open database
     let db = siamesedb::open_file(db_name)?;
-    let mut db_map = db.db_map_u64("some_list1")?;
+    let mut db_map = db.db_map_int("some_list1")?;
     let r = db_map.get_string(&120)?;
     assert_eq!(r, None);
-    db_map.put_string(120, "value120")?;
+    db_map.put_string(120.into(), "value120")?;
     let r = db_map.get_string(&120)?;
     assert_eq!(r, Some("value120".to_string()));
     db_map.sync_data()?;
@@ -69,7 +83,7 @@ fn main() -> std::io::Result<()> {
 ## Example Iterator:
 
 ```
-use siamesedb::{DbMapString, DbMap, DbXxx};
+use siamesedb::{DbMapDbString, DbMap, DbXxx, DbXxxBase, DbXxxObjectSafe};
 
 fn main() -> std::io::Result<()> {
     let db_name = "target/tmp/doc-test3.siamesedb";
@@ -97,14 +111,14 @@ fn main() -> std::io::Result<()> {
 }
 ```
 */
-use std::borrow::Borrow;
+use std::hash::Hash;
 use std::io::Result;
 use std::path::Path;
 
 pub mod filedb;
 pub mod memdb;
 
-pub use filedb::Bytes;
+pub use filedb::{DbBytes, DbInt, DbString};
 pub use filedb::{DbXxxIter, DbXxxIterMut};
 
 /// Open the memory db. This data is not stored in file.
@@ -117,25 +131,8 @@ pub fn open_file<P: AsRef<Path>>(path: P) -> Result<filedb::FileDb> {
     filedb::FileDb::open(path)
 }
 
-/// generic key-value map store interface. the key type is `KT`.
-pub trait DbXxx<KT: DbXxxKeyType> {
-    /// returns the value corresponding to the key.
-    fn get<Q>(&mut self, key: &Q) -> Result<Option<Vec<u8>>>
-    where
-        KT: Borrow<Q> + Ord,
-        Q: Ord + ?Sized;
-
-    /// inserts a key-value pair into the db.
-    fn put(&mut self, key: KT, value: &[u8]) -> Result<()>
-    where
-        KT: Ord;
-
-    /// removes a key from the db.
-    fn delete<Q>(&mut self, key: &Q) -> Result<()>
-    where
-        KT: Borrow<Q> + Ord,
-        Q: Ord + ?Sized;
-
+/// base interface for generic key-value map store interface. this is not include `KT`
+pub trait DbXxxBase {
     /// read and fill buffer.
     fn read_fill_buffer(&mut self) -> Result<()>;
 
@@ -147,28 +144,85 @@ pub trait DbXxx<KT: DbXxxKeyType> {
 
     /// synchronize data to storage, except file metadabe.
     fn sync_data(&mut self) -> Result<()>;
+}
 
-    /// returns true if the map contains a value for the specified key.
+/// generic key-value map store interface. the key type is `KT`. this is only object safe.
+pub trait DbXxxObjectSafe<KT: DbMapKeyType>: DbXxxBase {
+    /// returns the value corresponding to the key. this key is store raw data and type `&[u8]`.
+    fn get_kt(&mut self, key: &KT) -> Result<Option<Vec<u8>>>;
+
+    /// inserts a key-value pair into the db. this key is store raw data and type `&[u8]`.
+    fn put_kt(&mut self, key: &KT, value: &[u8]) -> Result<()>;
+
+    /// removes a key from the db. this key is store raw data and type `&[u8]`.
+    fn del_kt(&mut self, key: &KT) -> Result<Option<Vec<u8>>>;
+}
+
+/// generic key-value map store interface. the key type is `KT`.
+pub trait DbXxx<KT: DbMapKeyType>: DbXxxObjectSafe<KT> {
+    /// returns the value corresponding to the key.
     #[inline]
-    fn has_key<Q>(&mut self, key: &Q) -> Result<bool>
+    fn get<'a, Q>(&mut self, key: &'a Q) -> Result<Option<Vec<u8>>>
     where
-        KT: Borrow<Q> + Ord,
+        KT: From<&'a Q>,
         Q: Ord + ?Sized,
     {
-        self.get(key).map(|opt| opt.is_some())
+        let key_kt: KT = From::from(key);
+        self.get_kt(&key_kt)
     }
 
     /// returns the value corresponding to the key. the value is converted to `String`.
     #[inline]
-    fn get_string<Q>(&mut self, key: &Q) -> Result<Option<String>>
+    fn get_string<'a, Q>(&mut self, key: &'a Q) -> Result<Option<String>>
     where
-        KT: Borrow<Q> + Ord,
+        KT: From<&'a Q>,
         Q: Ord + ?Sized,
     {
         self.get(key)
             .map(|opt| opt.map(|val| String::from_utf8_lossy(&val).to_string()))
     }
 
+    /// gets bulk key-value paires from the db.
+    fn bulk_get<'a, Q>(&mut self, bulk_keys: &[&'a Q]) -> Result<Vec<Option<Vec<u8>>>>
+    where
+        KT: From<&'a Q>,
+        Q: Ord + ?Sized,
+    {
+        let mut result: Vec<(usize, Option<Vec<u8>>)> = Vec::new();
+        let mut vec: Vec<(usize, &Q)> =
+            bulk_keys.iter().enumerate().map(|(i, &a)| (i, a)).collect();
+        vec.sort_by(|a, b| b.1.cmp(a.1));
+        while let Some(ik) = vec.pop() {
+            let result_value = self.get(ik.1)?;
+            result.push((ik.0, result_value));
+        }
+        result.sort_by(|a, b| a.0.cmp(&(b.0)));
+        let ret: Vec<Option<Vec<u8>>> = result.iter().map(|a| a.1.clone()).collect();
+        Ok(ret)
+    }
+
+    /// gets bulk key-value paires from the db.
+    #[inline]
+    fn bulk_get_string<'a, Q>(&mut self, bulk_keys: &[&'a Q]) -> Result<Vec<Option<String>>>
+    where
+        KT: From<&'a Q>,
+        Q: Ord + ?Sized,
+    {
+        let vec = self.bulk_get(bulk_keys)?;
+        let mut ret = Vec::new();
+        for opt in vec {
+            let b = opt.map(|val| String::from_utf8_lossy(&val).to_string());
+            ret.push(b);
+        }
+        Ok(ret)
+    }
+
+    /// inserts a key-value pair into the db.
+    #[inline]
+    fn put(&mut self, key_kt: KT, value: &[u8]) -> Result<()> {
+        self.put_kt(&key_kt, value)
+    }
+
     /// inserts a key-value pair into the db-map. the value is `&str` and it is converted to `&[u8]`
     #[inline]
     fn put_string(&mut self, key: KT, value: &str) -> Result<()>
@@ -205,40 +259,55 @@ pub trait DbXxx<KT: DbXxxKeyType> {
         Ok(())
     }
 
-    /// gets bulk key-value paires from the db.
-    fn bulk_get<Q>(&mut self, bulk_keys: &[&Q]) -> Result<Vec<Option<Vec<u8>>>>
+    /// removes a key from the db.
+    #[inline]
+    fn delete<'a, Q>(&mut self, key: &'a Q) -> Result<Option<Vec<u8>>>
     where
-        KT: Borrow<Q> + Ord,
+        KT: From<&'a Q>,
+        Q: Ord + ?Sized,
+    {
+        let key_kt: KT = From::from(key);
+        self.del_kt(&key_kt)
+    }
+
+    /// removes a key from the db.
+    #[inline]
+    fn delete_string<'a, Q>(&mut self, key: &'a Q) -> Result<Option<String>>
+    where
+        KT: From<&'a Q>,
+        Q: Ord + ?Sized,
+    {
+        self.delete(key)
+            .map(|opt| opt.map(|val| String::from_utf8_lossy(&val).to_string()))
+    }
+
+    /// delete bulk key-value paires from the db.
+    fn bulk_delete<'a, Q>(&mut self, bulk_keys: &[&'a Q]) -> Result<Vec<Option<Vec<u8>>>>
+    where
+        KT: From<&'a Q>,
         Q: Ord + ?Sized,
     {
-        /*
-        let mut vec = Vec::with_capacity(bulk_keys.len());
-        for a in bulk_keys {
-            let value = self.get(a)?;
-            vec.push(value);
-        }
-        Ok(vec)
-        */
         let mut result: Vec<(usize, Option<Vec<u8>>)> = Vec::new();
         let mut vec: Vec<(usize, &Q)> =
             bulk_keys.iter().enumerate().map(|(i, &a)| (i, a)).collect();
         vec.sort_by(|a, b| b.1.cmp(a.1));
         while let Some(ik) = vec.pop() {
-            let result_value = self.get(ik.1)?;
+            let result_value = self.delete(ik.1)?;
             result.push((ik.0, result_value));
         }
         result.sort_by(|a, b| a.0.cmp(&(b.0)));
         let ret: Vec<Option<Vec<u8>>> = result.iter().map(|a| a.1.clone()).collect();
         Ok(ret)
     }
-    /// gets bulk key-value paires from the db.
+
+    /// delete bulk key-value paires from the db.
     #[inline]
-    fn bulk_get_string<Q>(&mut self, bulk_keys: &[&Q]) -> Result<Vec<Option<String>>>
+    fn bulk_delete_string<'a, Q>(&mut self, bulk_keys: &[&'a Q]) -> Result<Vec<Option<String>>>
     where
-        KT: Borrow<Q> + Ord,
+        KT: From<&'a Q>,
         Q: Ord + ?Sized,
     {
-        let vec = self.bulk_get(bulk_keys)?;
+        let vec = self.bulk_delete(bulk_keys)?;
         let mut ret = Vec::new();
         for opt in vec {
             let b = opt.map(|val| String::from_utf8_lossy(&val).to_string());
@@ -249,29 +318,39 @@ pub trait DbXxx<KT: DbXxxKeyType> {
 }
 
 /// key-value db map store interface.
-pub trait DbMap<KT: DbXxxKeyType>: DbXxx<KT> {
+pub trait DbMap<KT: DbMapKeyType>: DbXxx<KT> {
     fn iter(&self) -> DbXxxIter<KT>;
     fn iter_mut(&mut self) -> DbXxxIterMut<KT>;
 }
 
 /// key-value map store interface. the key type is `String`.
-pub trait DbMapString: DbXxx<String> {}
+pub trait DbMapDbString: DbXxx<DbString> {}
 
 /// key-value map store interface. the key type is `u64`.
-pub trait DbMapU64: DbXxx<u64> {}
+pub trait DbMapDbInt: DbXxx<DbInt> {}
 
 /// key-value map store interface. the key type is `Vec<u8>`.
-pub trait DbMapBytes: DbXxx<Bytes> {}
+pub trait DbMapDbBytes: DbXxx<DbBytes> {}
 
 /// key type
-pub trait DbXxxKeyType: Ord + Clone + Default {
-    /// Signature of database file.
+pub trait DbMapKeyType: Ord + Clone + Default + HashValue {
+    /// Convert a byte slice to Key.
+    fn from_bytes(bytes: &[u8]) -> Self;
+    /// Signature in header of database file.
     fn signature() -> [u8; 8];
-    fn as_bytes(&self) -> Vec<u8>;
-    /// Converts a KeyType into a byte vector.
-    //fn into_bytes(self) -> Vec<u8>;
-    fn from(bytes: &[u8]) -> Self;
-    fn byte_len(&self) -> usize {
-        self.as_bytes().len()
+    /// Byte slice of data to be saved
+    fn as_bytes(&self) -> &[u8];
+    /// Compare with stored data
+    fn cmp_u8(&self, other: &[u8]) -> std::cmp::Ordering;
+}
+
+/// hash value for htx
+pub trait HashValue: Hash {
+    /// hash value for htx
+    fn hash_value(&self) -> u64 {
+        use std::hash::Hasher;
+        let mut hasher = std::collections::hash_map::DefaultHasher::new();
+        self.hash(&mut hasher);
+        hasher.finish()
     }
 }
