@@ -1,6 +1,6 @@
 use siamesedb::filedb::CheckFileDbMap;
-use siamesedb::filedb::FileDbMapString;
-use siamesedb::DbXxx;
+use siamesedb::filedb::FileDbMapDbString;
+use siamesedb::{DbXxx, DbXxxBase};
 
 fn main() -> std::io::Result<()> {
     let args: Vec<String> = std::env::args().collect();
@@ -15,8 +15,8 @@ fn test01(db_name: &str) -> std::io::Result<()> {
     //
     let val = "ab".repeat(1024 * 1024 * 1024);
     //
-    db_map.put_string("A".to_string(), &val).unwrap();
-    db_map.put_string("B".to_string(), &val).unwrap();
+    db_map.put_string("A".into(), &val).unwrap();
+    db_map.put_string("B".into(), &val).unwrap();
     //
     db_map.sync_data().unwrap();
     _print_check_db_map(
@@ -37,13 +37,21 @@ struct CheckC {
     f_graph: bool,
 }
 
-fn _print_check_db_map(db_map: &FileDbMapString, check_cnf: CheckC) {
+fn _print_check_db_map(db_map: &FileDbMapDbString, check_cnf: CheckC) {
     if check_cnf.f_graph {
         println!("{}", db_map.graph_string_with_key_string().unwrap());
     }
-    println!("record free: {:?}", db_map.count_of_free_record().unwrap());
-    let (rec_v, node_v) = db_map.count_of_used_node().unwrap();
-    println!("record used: {:?}", rec_v);
+    println!(
+        "key record free: {:?}",
+        db_map.count_of_free_key_record().unwrap()
+    );
+    let (key_rec_v, val_rec_v, node_v) = db_map.count_of_used_node().unwrap();
+    println!("key record used: {:?}", key_rec_v);
+    println!(
+        "value record free: {:?}",
+        db_map.count_of_free_value_record().unwrap()
+    );
+    println!("value record used: {:?}", val_rec_v);
     println!("node free: {:?}", db_map.count_of_free_node().unwrap());
     println!("node used: {:?}", node_v);
     println!("db_map.is_balanced(): {}", db_map.is_balanced().unwrap());
@@ -58,7 +66,11 @@ fn _print_check_db_map(db_map: &FileDbMapString, check_cnf: CheckC) {
     #[cfg(feature = "buf_stats")]
     println!("db_map.buf_stats(): {:?}", db_map.buf_stats());
     println!(
-        "record_size_stats(): {}",
-        db_map.record_size_stats().unwrap()
+        "key_record_size_stats(): {}",
+        db_map.key_record_size_stats().unwrap()
+    );
+    println!(
+        "value_record_size_stats(): {}",
+        db_map.value_record_size_stats().unwrap()
     );
 }
