@@ -1,6 +1,6 @@
 use siamesedb::filedb::CheckFileDbMap;
-use siamesedb::filedb::FileDbMapString;
-use siamesedb::DbXxx;
+use siamesedb::filedb::FileDbMapDbString;
+use siamesedb::{DbXxx, DbXxxBase};
 
 fn main() {
     test_fixtures_fruits();
@@ -8,7 +8,7 @@ fn main() {
 
 fn do_file_map_string<F>(db_name: &str, mut fun: F)
 where
-    F: FnMut(FileDbMapString),
+    F: FnMut(FileDbMapDbString),
 {
     let db = siamesedb::open_file(db_name).unwrap();
     let db_map = db.db_map_string("some_map1").unwrap();
@@ -49,54 +49,54 @@ fn test_fixtures_fruits() {
     let data = load_fixtures(concat!(base_path!(0), "fixtures/test-fruits.txt"));
     let data = &data[..5000];
     //
-    do_file_map_string(db_name, |mut db_map: FileDbMapString| {
+    do_file_map_string(db_name, |mut db_map: FileDbMapDbString| {
         for (k, v) in data {
-            db_map.put(k.clone(), v.as_bytes()).unwrap();
+            db_map.put(k.into(), v.as_bytes()).unwrap();
         }
         //
         db_map.sync_data().unwrap();
     });
     //
-    do_file_map_string(db_name, |db_map: FileDbMapString| {
+    do_file_map_string(db_name, |db_map: FileDbMapDbString| {
         assert!(db_map.is_balanced().unwrap());
         assert!(db_map.is_mst_valid().unwrap());
         assert!(db_map.is_dense().unwrap());
     });
     //
-    do_file_map_string(db_name, |mut db_map: FileDbMapString| {
+    do_file_map_string(db_name, |mut db_map: FileDbMapDbString| {
         for (k, v) in data {
-            db_map.put(k.clone(), v.as_bytes()).unwrap();
+            db_map.put(k.into(), v.as_bytes()).unwrap();
         }
         //
         db_map.sync_data().unwrap();
     });
     //
-    do_file_map_string(db_name, |db_map: FileDbMapString| {
+    do_file_map_string(db_name, |db_map: FileDbMapDbString| {
         assert!(db_map.is_balanced().unwrap());
         assert!(db_map.is_mst_valid().unwrap());
         assert!(db_map.is_dense().unwrap());
     });
     //
-    do_file_map_string(db_name, |mut db_map: FileDbMapString| {
+    do_file_map_string(db_name, |mut db_map: FileDbMapDbString| {
         db_map
-            .put_string("9909909900".to_string(), "TEST, v9909909900")
+            .put_string("9909909900".into(), "TEST, v9909909900")
             .unwrap();
         db_map
-            .put_string("9909909901".to_string(), "TEST, v9909909901")
+            .put_string("9909909901".into(), "TEST, v9909909901")
             .unwrap();
         db_map
-            .put_string("9909909902".to_string(), "TEST, v9909909902")
+            .put_string("9909909902".into(), "TEST, v9909909902")
             .unwrap();
         db_map.sync_data().unwrap();
     });
     //
-    do_file_map_string(db_name, |db_map: FileDbMapString| {
+    do_file_map_string(db_name, |db_map: FileDbMapDbString| {
         assert!(db_map.is_balanced().unwrap());
         assert!(db_map.is_mst_valid().unwrap());
         assert!(db_map.is_dense().unwrap());
     });
     //
-    do_file_map_string(db_name, |mut db_map: FileDbMapString| {
+    do_file_map_string(db_name, |mut db_map: FileDbMapDbString| {
         assert_eq!(
             db_map.get_string("9909909900").unwrap(),
             Some("TEST, v9909909900".to_string())
@@ -111,26 +111,26 @@ fn test_fixtures_fruits() {
         );
     });
     //
-    do_file_map_string(db_name, |mut db_map: FileDbMapString| {
+    do_file_map_string(db_name, |mut db_map: FileDbMapDbString| {
         db_map.delete("9909909900").unwrap();
         db_map.delete("9909909901").unwrap();
         db_map.delete("9909909902").unwrap();
         db_map.sync_data().unwrap();
     });
     /*
-    do_file_map_string(db_name, |db_map: FileDbMapString| {
+    do_file_map_string(db_name, |db_map: FileDbMapDbString| {
         println!("{}", db_map.to_graph_string_with_key_string().unwrap());
     });
     return;
     */
     //
-    do_file_map_string(db_name, |db_map: FileDbMapString| {
+    do_file_map_string(db_name, |db_map: FileDbMapDbString| {
         assert!(db_map.is_balanced().unwrap());
         assert!(db_map.is_mst_valid().unwrap());
         assert!(db_map.is_dense().unwrap());
     });
     //
-    do_file_map_string(db_name, |mut db_map: FileDbMapString| {
+    do_file_map_string(db_name, |mut db_map: FileDbMapDbString| {
         assert_eq!(db_map.get_string("9909909900").unwrap(), None);
         assert_eq!(db_map.get_string("9909909901").unwrap(), None);
         assert_eq!(db_map.get_string("9909909902").unwrap(), None);
@@ -145,15 +145,15 @@ fn test_fixtures_fruits() {
         );
     });
     //
-    do_file_map_string(db_name, |mut db_map: FileDbMapString| {
+    do_file_map_string(db_name, |mut db_map: FileDbMapDbString| {
         db_map
-            .put_string("9909909900".to_string(), "TEST, v9909909900")
+            .put_string("9909909900".into(), "TEST, v9909909900")
             .unwrap();
         db_map
-            .put_string("9909909901".to_string(), "TEST, v9909909901")
+            .put_string("9909909901".into(), "TEST, v9909909901")
             .unwrap();
         db_map
-            .put_string("9909909902".to_string(), "TEST, v9909909902")
+            .put_string("9909909902".into(), "TEST, v9909909902")
             .unwrap();
         db_map.sync_data().unwrap();
         _print_check_db_map(
@@ -165,24 +165,24 @@ fn test_fixtures_fruits() {
         );
     });
     //
-    do_file_map_string(db_name, |db_map: FileDbMapString| {
+    do_file_map_string(db_name, |db_map: FileDbMapDbString| {
         assert!(db_map.is_balanced().unwrap());
         assert!(db_map.is_mst_valid().unwrap());
         assert!(db_map.is_dense().unwrap());
     });
     //
-    do_file_map_string(db_name, |mut db_map: FileDbMapString| {
+    do_file_map_string(db_name, |mut db_map: FileDbMapDbString| {
         assert_eq!(
             db_map.get_string("9909909900").unwrap(),
-            Some("TEST, v9909909900".to_string())
+            Some("TEST, v9909909900".into())
         );
         assert_eq!(
             db_map.get_string("9909909901").unwrap(),
-            Some("TEST, v9909909901".to_string())
+            Some("TEST, v9909909901".into())
         );
         assert_eq!(
             db_map.get_string("9909909902").unwrap(),
-            Some("TEST, v9909909902".to_string())
+            Some("TEST, v9909909902".into())
         );
     });
 }
@@ -195,13 +195,21 @@ struct CheckC {
     f_graph: bool,
 }
 
-fn _print_check_db_map(db_map: &FileDbMapString, check_cnf: CheckC) {
+fn _print_check_db_map(db_map: &FileDbMapDbString, check_cnf: CheckC) {
     if check_cnf.f_graph {
         println!("{}", db_map.graph_string_with_key_string().unwrap());
     }
-    println!("record free: {:?}", db_map.count_of_free_record().unwrap());
-    let (rec_v, node_v) = db_map.count_of_used_node().unwrap();
-    println!("record used: {:?}", rec_v);
+    println!(
+        "key record free: {:?}",
+        db_map.count_of_free_key_record().unwrap()
+    );
+    let (key_rec_v, val_rec_v, node_v) = db_map.count_of_used_node().unwrap();
+    println!("key record used: {:?}", key_rec_v);
+    println!(
+        "value record free: {:?}",
+        db_map.count_of_free_value_record().unwrap()
+    );
+    println!("value record used: {:?}", val_rec_v);
     println!("node free: {:?}", db_map.count_of_free_node().unwrap());
     println!("node used: {:?}", node_v);
     println!("db_map.is_balanced(): {}", db_map.is_balanced().unwrap());
@@ -216,7 +224,11 @@ fn _print_check_db_map(db_map: &FileDbMapString, check_cnf: CheckC) {
     #[cfg(feature = "buf_stats")]
     println!("db_map.buf_stats(): {:?}", db_map.buf_stats());
     println!(
-        "record_size_stats(): {}",
-        db_map.record_size_stats().unwrap()
+        "key_record_size_stats(): {}",
+        db_map.key_record_size_stats().unwrap()
+    );
+    println!(
+        "value_record_size_stats(): {}",
+        db_map.value_record_size_stats().unwrap()
     );
 }
